==> spirit_hub-0.0.1.13.tar.gz: spirit_hub-0.0.1.13/app/controllers/spirit_hub/tagsets_controller.rb <==
# frozen_string_literal: true

module SpiritHub
  class TagsetsController < ApplicationController
    include StateForm

    before_action :skip_policy_scope, only: %i[add_doc_form add_doc form]
    before_action :find_model, except: %i[search_directories form bulk_delete classify]
    before_action :find_models_with_ids, only: %i[form bulk_delete classify]

    def journal_entries
      authorize(@model, policy_class:)

      @journal_entries = @model.journal_entries.order_by(created_at: :asc)

      respond_to(&:html)
    end

    def doc
      authorize(@model, policy_class:)

      @journal_entries = @model.journal_entries.order_by(created_at: :asc)
      @doc = @model.docs.find_by(id: params[:doc_id])

      respond_to(&:html)
    end

    def add_doc_form
      authorize(Tagset, policy_class:)

      tagsets = Tagset.where(type: 'directory', spirit_hubdoc: true).order_by(updated_at: :desc).limit(5)

      render turbo_stream: [
        turbo_stream.update(:modal, partial: 'spirit_hub/tagsets/add_doc_form', locals: { tagsets: })
      ]
    end

    def add_doc
      authorize(Tagset, policy_class:)

      files = add_doc_params[:files]
      regroup_files = add_doc_params[:regroup_files]
      tagset_id = add_doc_params[:tagset_id]

      models = process_files(files, tagset_id, regroup_files)

      flash.now[:notice] = 'Document ajouté'
      update_flash_message

      turbo_action_for_add_doc(models)
      add_action(models, add_doc_params)

      render turbo_stream: @turbo_actions
    end

    def share
      authorize(@model, policy_class:)

      SpiritHub::TagsetShare.call(tagset: @model, **share_params)

      @turbo_actions << turbo_stream.replace("informed_badges_#{@model.id}", partial: 'spirit_hub/common/informed_badges', locals: { model: @model })

      flash.now[:notice] = 'Document partagé'
      update_flash_message

      update_turbo_action_current_state(@model)
      add_action(@model, share_params)

      render turbo_stream: @turbo_actions
    end

    def respond
      authorize(@model, policy_class:)

      respond_params = share_params.merge!(action: :respond)
      SpiritHub::TagsetShare.call(tagset: @model, **respond_params)

      flash.now[:notice] = 'Réponse envoyée'
      update_flash_message

      update_turbo_action_current_state(@model)
      add_action(@model, respond_params)

      render turbo_stream: @turbo_actions
    end

    def request_signature
      authorize(@model, policy_class:)

      success = SpiritHub::TagsetRequestSignature.call(tagset: @model, **request_signature_params)

      if success
        flash.now[:notice] = 'Signature requise'
      else
        flash.now[:alert] = "Quelque chose s'est mal passé. Réessayez"
      end

      update_flash_message

      update_turbo_action_current_state(@model)
      add_action(@model, request_signature_params)

      render turbo_stream: @turbo_actions
    end

    def classify
      @models.each do |model|
        authorize(model, policy_class:)

        model.add_tagset_doc_to_object(classify_params[:tagset_id], @user)
      end

      if route_controller_from_referer == 'spirit_hub/directories'
        @turbo_actions = [turbo_stream.reload] # too compelexe to do a selected update
      else
        flash.now[:notice] = 'Documents classés'
        update_flash_message
        classify_turbo_actions(@models)
        add_action(@models, classify_params)
      end

      render turbo_stream: @turbo_actions
    end

    def search_directories
      authorize(Tagset, policy_class:)

      tagsets = policy_scope(Tagset, policy_scope_class:).where(
        type: 'directory',
        title: /#{Regexp.escape(params[:query]).accent_to_regexp}/i
      ).order_by(updated_at: :desc).limit(5)

      respond_to do |format|
        format.turbo_stream do
          render turbo_stream: [
            turbo_stream.replace(:directory_selector, partial: 'spirit_hub/tagsets/directory_selector', locals: { tagsets: })
          ]
        end
      end
    end

    def add_favorite
      authorize(@model, policy_class:)

      if Favorite.create(user: @user, tagset: @model)
        set_user_favorites
        flash.now[:notice] = 'Ajouté aux favoris'

        render turbo_stream: [
          turbo_stream.replace("add_remove_favorites_#{@model.id}", partial: 'spirit_hub/common/add_remove_favorites', locals: { model: @model }),
          turbo_stream.prepend(:flash_messages, partial: 'spirit_hub/common/flash_messages')
        ]
      else
        flash.now[:alert] = 'Erreur lors de l\'ajout au favoris'

        render turbo_stream: [
          turbo_stream.prepend(:flash_messages, partial: 'spirit_hub/common/flash_messages')
        ]
      end
    end

    def remove_favorite
      authorize(@model, policy_class:)

      if Favorite.where(user: @user, tagset: @model).destroy_all
        set_user_favorites
        flash.now[:notice] = 'Supprimé des favoris'

        render turbo_stream: [
          turbo_stream.replace("add_remove_favorites_#{@model.id}", partial: 'spirit_hub/common/add_remove_favorites', locals: { model: @model }),
          turbo_stream.prepend(:flash_messages, partial: 'spirit_hub/common/flash_messages')
        ]
      else
        flash.now[:alert] = 'Erreur lors de l\'ajout au favoris'

        render turbo_stream: [
          turbo_stream.prepend(:flash_messages, partial: 'spirit_hub/common/flash_messages')
        ]
      end
    end

    def delete_item
      authorize(@model, policy_class:)

      if SpiritHub::TagsetPurger.call(@model, @user)
        flash.now[:notice] = @model.doc? ? 'Document supprimé' : 'Dossier supprimé'

        render turbo_stream: [
          turbo_stream.remove("tagset_#{params[:id]}"),
          turbo_stream.prepend(:flash_messages, partial: 'spirit_hub/common/flash_messages'),
          turbo_stream.custom_action(:change_counter, target: :end_index, attributes: { amount: -1 })
        ]
      else
        flash.now[:alert] = 'Erreur lors de la suppresion'

        render turbo_stream: [
          turbo_stream.prepend(:flash_messages, partial: 'spirit_hub/common/flash_messages')
        ]
      end
    end

    def bulk_delete
      ids = params[:ids].split(',')

      @models.each do |tagset|
        authorize(tagset, policy_class:)
      end

      @models.each do |tagset| # rubocop:disable Style/CombinableLoops
        SpiritHub::TagsetPurger.call(tagset, @user)
      end

      flash.now[:notice] = 'Documents supprimés'

      render turbo_stream: [
        *ids.map { |id| turbo_stream.remove("tagset_#{id}") },
        turbo_stream.prepend(:flash_messages, partial: 'spirit_hub/common/flash_messages'),
        turbo_stream.custom_action(:change_counter, target: :end_index, attributes: { amount: -ids.length })
      ]
    end

    def form
      @models.each do |model|
        authorize(model, policy_class:)
      end

      generate_action_form(form_params[:state_key].to_sym, @models)

      render turbo_stream: @turbo_actions
    end

    def evaluate
      authorize(@model, policy_class:)
      state = State.find(evaluate_params[:state_id])

      SpiritHub::TagsetSendToUsers.call(state_key: state.key, tagset: @model, **evaluate_params)

      flash.now[:notice] = state.key == :refuse ? 'Document refusé' : 'Document validé'
      update_flash_message

      validate_or_refuse_turbo_actions(@model)
      add_action(@model, evaluate_params)

      render turbo_stream: @turbo_actions
    end

    def send_to
      authorize(@model, policy_class:)

      state = State.find(send_to_params[:state_id])

      SpiritHub::TagsetSendToUsers.call(state_key: state.key, tagset: @model, **send_to_params)

      flash.now[:notice] = state.title&.downcase&.gsub('envoyer', 'envoyé')&.capitalize
      update_flash_message

      send_to_turbo_actions(@model)

      render turbo_stream: @turbo_actions
    end

    def inline_edit
      authorize(@model, policy_class:)

      key = params[:key].to_sym
      options = User.all.map { |u| [u.full_name, u.id] }

      render turbo_stream: turbo_stream.update("inline_edit_#{key}_#{@model.id}", partial: 'spirit_hub/tagsets/inline_edit', locals: { model: @model, options:, key: })
    end

    def update_inline
      authorize(@model, policy_class:)

      key = inline_update_params[:key].to_sym

      @model.update(key => inline_update_params[key])
      value = find_inline_value(@model, key)
      SpiritHub::JournalEntry.edit_action_log('DIRECTORY_INFORMATION_UPDATED', tagset: @model, user: @user)

      flash.now[:notice] = 'Valeur mise à jour'
      update_flash_message

      @turbo_actions << turbo_stream.replace("inline_value_#{key}_#{@model.id}", partial: 'spirit_hub/common/inline_value', locals: { model: @model, value:, key: })

      render turbo_stream: @turbo_actions
    end

    private

    def find_model
      @model = policy_scope(Tagset, policy_scope_class:).find_by(id: params[:id])
    end

    def find_models_with_ids
      @models = policy_scope(Tagset, policy_scope_class:).where(:id.in => params[:ids].split(','))
    end

    def share_params
      formatted_params(
        :id, :subject, :message, :next_state_id, :add_action, :authenticity_token,
        to: [], cc: [], bcc: [], attached_tagset_ids: [], attachments: []
      )
    end

    def evaluate_params
      formatted_params(:id, :message, :state_id, :next_state_id, :add_action, :authenticity_token, recipients: [])
    end

    def classify_params
      formatted_params(:tagset_id, :ids, :next_state_id, :add_action, :authenticity_token)
    end

    def add_doc_params
      formatted_params(:tagset_id, :next_state_id, :add_action, :authenticity_token, :regroup_files, files: [])
    end

    def form_params
      formatted_params(:state_key, :ids, :authenticity_token)
    end

    def inline_update_params
      params.require(:tagset).permit(:key, :responsible, :start_date, :end_date, accounted: [])
    end

    def send_to_params
      formatted_params(:id, :state_id, :message, :authenticity_token, recipients: [])
    end

    def find_inline_value(model, key)
      if %i[start_date end_date].include?(key)
        model.send(key)&.strftime('%d/%m/%Y')
      else
        SpiritHub::ActorFormatter.actor_ids_to_sentence(model.send(key), without_indicator: true)
      end
    end

    def request_signature_params
      formatted_params(:id, :authenticity_token, :doc_id, :comments, recipients: [])
    end

    def process_files(files, tagset_id, regroup_files)
      return [create_tagset_with_file(files, tagset_id)] if files.length < 2 || regroup_files == 'true'

      files.map { |file| create_tagset_with_file(file, tagset_id) }
    end

    def create_tagset_with_file(file, tagset_id)
      model = Tagset.create_tagset_doc_from_file(file, @user)
      model.add_tagset_doc_to_object(tagset_id, @user)
      model
    end

    def validate_or_refuse_turbo_actions(model)
      case route_controller_from_referer
      when 'spirit_hub/todos'
        if !model.is_accounted?(@user.id.to_s)
          @turbo_actions << turbo_stream.remove("tagset_#{model.id}")
        else
          update_turbo_action_current_state(model)
        end
      when 'spirit_hub/tagsets'
        @turbo_actions << turbo_stream.remove('doc_validate_refuse')
      else
        update_turbo_action_current_state(model)
      end
    end

    def send_to_turbo_actions(model)
      if route_controller_from_referer == 'spirit_hub/todos' && !model.is_accounted?(@user.id.to_s)
        @turbo_actions << turbo_stream.remove("tagset_#{model.id}")
      else
        update_turbo_action_current_state(model)
      end
    end

    def classify_turbo_actions(models)
      case route_controller_from_referer
      when 'spirit_hub/to_classify'
        models.each do |model|
          @turbo_actions << turbo_stream.remove("tagset_#{model.id}")
        end
        @turbo_actions << turbo_stream.custom_action(:change_counter, target: :end_index, attributes: { amount: -models.length })
      else
        models.each do |model|
          update_turbo_action_current_state(model)
        end
      end
    end

    def turbo_action_for_add_doc(models)
      case route_controller_from_referer
      when 'spirit_hub/favorites'
        @counts = SpiritHub::FavoritesCounts.new(@user).counts
        @turbo_actions << turbo_stream.replace('favorites-counts', partial: 'spirit_hub/favorites/counts', formats: [:html])
      when 'spirit_hub/todos'
        models.each { |model| handle_todos_add_doc(model) }
      when 'spirit_hub/to_classify'
        models.each { |model| handle_to_classify_add_doc(model) }
      else
        []
      end
    end

    def handle_todos_add_doc(model)
      return [] if model.reload.parent_tagset_ids.blank?

      # TODO : Refactor this
      referer_controller = SpiritHub::TodosController.new
      referer_controller.instance_variable_set(:@user, @user)
      referer_controller.set_columns
      @user_column_names = referer_controller.instance_variable_get(:@user_column_names)
      @user_favorites = []

      partial = referer_controller.set_view_type == 'table' ? 'spirit_hub/todos/row' : 'spirit_hub/todos/card'
      @turbo_actions << turbo_stream.prepend('todos-table-body', partial:, formats: [:html], locals: { model: })
    end

    def handle_to_classify_add_doc(model)
      return [] unless model.reload.parent_tagset_ids.blank?

      # TODO : Refactor this
      referer_controller = SpiritHub::TodosController.new
      referer_controller.instance_variable_set(:@user, @user)
      referer_controller.set_columns
      @user_column_names = referer_controller.instance_variable_get(:@user_column_names)
      @user_favorites = []

      partial = referer_controller.set_view_type == 'table' ? 'spirit_hub/to_classify/row' : 'spirit_hub/to_classify/card'
      @turbo_actions << turbo_stream.prepend('to-classify-table-body', partial:, formats: [:html], locals: { model: })
    end
  end
end

==> spirit_hub-0.0.1.14.tar.gz: spirit_hub-0.0.1.14/app/controllers/spirit_hub/tagsets_controller.rb <==
# frozen_string_literal: true

module SpiritHub
  class TagsetsController < ApplicationController
    include StateForm

    before_action :skip_policy_scope, only: %i[add_doc_form add_doc form]
    before_action :find_model, except: %i[search_directories form bulk_delete classify add_doc_form add_doc]
    before_action :find_models_with_ids, only: %i[form bulk_delete classify]

    def journal_entries
      authorize(@model, policy_class:)

      @journal_entries = @model.journal_entries.order_by(created_at: :asc)

      respond_to(&:html)
    end

    def doc
      authorize(@model, policy_class:)

      @journal_entries = @model.journal_entries.order_by(created_at: :asc)
      @doc = @model.docs.find_by(id: params[:doc_id])

      respond_to(&:html)
    end

    def add_doc_form
      authorize(Tagset, policy_class:)

      tagsets = Tagset.where(type: 'directory', spirit_hubdoc: true).order_by(updated_at: :desc).limit(5)

      render turbo_stream: [
        turbo_stream.update(:modal, partial: 'spirit_hub/tagsets/add_doc_form', locals: { tagsets: })
      ]
    end

    def add_doc
      authorize(Tagset, policy_class:)

      files = add_doc_params[:files]
      regroup_files = add_doc_params[:regroup_files]
      tagset_id = add_doc_params[:tagset_id]

      models = process_files(files, tagset_id, regroup_files)

      flash.now[:notice] = 'Document ajouté'
      update_flash_message

      turbo_action_for_add_doc(models)
      add_action(models, add_doc_params)

      render turbo_stream: @turbo_actions
    end

    def share
      authorize(@model, policy_class:)

      SpiritHub::TagsetShare.call(tagset: @model, **share_params)

      @turbo_actions << turbo_stream.replace("informed_badges_#{@model.id}", partial: 'spirit_hub/common/informed_badges', locals: { model: @model })

      flash.now[:notice] = 'Document partagé'
      update_flash_message

      update_turbo_action_current_state(@model)
      add_action(@model, share_params)

      render turbo_stream: @turbo_actions
    end

    def respond
      authorize(@model, policy_class:)

      respond_params = share_params.merge!(action: :respond)
      SpiritHub::TagsetShare.call(tagset: @model, **respond_params)

      flash.now[:notice] = 'Réponse envoyée'
      update_flash_message

      update_turbo_action_current_state(@model)
      add_action(@model, respond_params)

      render turbo_stream: @turbo_actions
    end

    def request_signature
      authorize(@model, policy_class:)

      success = SpiritHub::TagsetRequestSignature.call(tagset: @model, **request_signature_params)

      if success
        flash.now[:notice] = 'Signature requise'
      else
        flash.now[:alert] = "Quelque chose s'est mal passé. Réessayez"
      end

      update_flash_message

      update_turbo_action_current_state(@model)
      add_action(@model, request_signature_params)

      render turbo_stream: @turbo_actions
    end

    def classify
      @models.each do |model|
        authorize(model, policy_class:)

        model.add_tagset_doc_to_object(classify_params[:tagset_id], @user)
      end

      if route_controller_from_referer == 'spirit_hub/directories'
        @turbo_actions = [turbo_stream.reload] # too compelexe to do a selected update
      else
        flash.now[:notice] = 'Documents classés'
        update_flash_message
        classify_turbo_actions(@models)
        add_action(@models, classify_params)
      end

      render turbo_stream: @turbo_actions
    end

    def search_directories
      authorize(Tagset, policy_class:)

      tagsets = policy_scope(Tagset, policy_scope_class:).where(
        type: 'directory',
        title: /#{Regexp.escape(params[:query]).accent_to_regexp}/i
      ).order_by(updated_at: :desc).limit(5)

      respond_to do |format|
        format.turbo_stream do
          render turbo_stream: [
            turbo_stream.replace(:directory_selector, partial: 'spirit_hub/tagsets/directory_selector', locals: { tagsets: })
          ]
        end
      end
    end

    def add_favorite
      authorize(@model, policy_class:)

      if Favorite.create(user: @user, tagset: @model)
        set_user_favorites
        flash.now[:notice] = 'Ajouté aux favoris'

        render turbo_stream: [
          turbo_stream.replace("add_remove_favorites_#{@model.id}", partial: 'spirit_hub/common/add_remove_favorites', locals: { model: @model }),
          turbo_stream.prepend(:flash_messages, partial: 'spirit_hub/common/flash_messages')
        ]
      else
        flash.now[:alert] = 'Erreur lors de l\'ajout au favoris'

        render turbo_stream: [
          turbo_stream.prepend(:flash_messages, partial: 'spirit_hub/common/flash_messages')
        ]
      end
    end

    def remove_favorite
      authorize(@model, policy_class:)

      if Favorite.where(user: @user, tagset: @model).destroy_all
        set_user_favorites
        flash.now[:notice] = 'Supprimé des favoris'

        render turbo_stream: [
          turbo_stream.replace("add_remove_favorites_#{@model.id}", partial: 'spirit_hub/common/add_remove_favorites', locals: { model: @model }),
          turbo_stream.prepend(:flash_messages, partial: 'spirit_hub/common/flash_messages')
        ]
      else
        flash.now[:alert] = 'Erreur lors de l\'ajout au favoris'

        render turbo_stream: [
          turbo_stream.prepend(:flash_messages, partial: 'spirit_hub/common/flash_messages')
        ]
      end
    end

    def delete_item
      authorize(@model, policy_class:)

      if SpiritHub::TagsetPurger.call(@model, @user)
        flash.now[:notice] = @model.doc? ? 'Document supprimé' : 'Dossier supprimé'

        render turbo_stream: [
          turbo_stream.remove("tagset_#{params[:id]}"),
          turbo_stream.prepend(:flash_messages, partial: 'spirit_hub/common/flash_messages'),
          turbo_stream.custom_action(:change_counter, target: :end_index, attributes: { amount: -1 })
        ]
      else
        flash.now[:alert] = 'Erreur lors de la suppresion'

        render turbo_stream: [
          turbo_stream.prepend(:flash_messages, partial: 'spirit_hub/common/flash_messages')
        ]
      end
    end

    def bulk_delete
      ids = params[:ids].split(',')

      @models.each do |tagset|
        authorize(tagset, policy_class:)
      end

      @models.each do |tagset| # rubocop:disable Style/CombinableLoops
        SpiritHub::TagsetPurger.call(tagset, @user)
      end

      flash.now[:notice] = 'Documents supprimés'

      render turbo_stream: [
        *ids.map { |id| turbo_stream.remove("tagset_#{id}") },
        turbo_stream.prepend(:flash_messages, partial: 'spirit_hub/common/flash_messages'),
        turbo_stream.custom_action(:change_counter, target: :end_index, attributes: { amount: -ids.length })
      ]
    end

    def form
      @models.each do |model|
        authorize(model, policy_class:)
      end

      generate_action_form(form_params[:state_key].to_sym, @models)

      render turbo_stream: @turbo_actions
    end

    def evaluate
      authorize(@model, policy_class:)
      state = State.find(evaluate_params[:state_id])

      SpiritHub::TagsetSendToUsers.call(state_key: state.key, tagset: @model, **evaluate_params)

      flash.now[:notice] = state.key == :refuse ? 'Document refusé' : 'Document validé'
      update_flash_message

      validate_or_refuse_turbo_actions(@model)
      add_action(@model, evaluate_params)

      render turbo_stream: @turbo_actions
    end

    def send_to
      authorize(@model, policy_class:)

      state = State.find(send_to_params[:state_id])

      SpiritHub::TagsetSendToUsers.call(state_key: state.key, tagset: @model, **send_to_params)

      flash.now[:notice] = state.title&.downcase&.gsub('envoyer', 'envoyé')&.capitalize
      update_flash_message

      send_to_turbo_actions(@model)

      render turbo_stream: @turbo_actions
    end

    def inline_edit
      authorize(@model, policy_class:)

      key = params[:key].to_sym
      options = User.all.map { |u| [u.full_name, u.id] }

      render turbo_stream: turbo_stream.update("inline_edit_#{key}_#{@model.id}", partial: 'spirit_hub/tagsets/inline_edit', locals: { model: @model, options:, key: })
    end

    def update_inline
      authorize(@model, policy_class:)

      key = inline_update_params[:key].to_sym

      @model.update(key => inline_update_params[key])
      value = find_inline_value(@model, key)
      SpiritHub::JournalEntry.edit_action_log('DIRECTORY_INFORMATION_UPDATED', tagset: @model, user: @user)

      flash.now[:notice] = 'Valeur mise à jour'
      update_flash_message

      @turbo_actions << turbo_stream.replace("inline_value_#{key}_#{@model.id}", partial: 'spirit_hub/common/inline_value', locals: { model: @model, value:, key: })

      render turbo_stream: @turbo_actions
    end

    private

    def find_model
      @model = policy_scope(Tagset, policy_scope_class:).find_by!(id: params[:id])
    end

    def find_models_with_ids
      @models = policy_scope(Tagset, policy_scope_class:).where(:id.in => params[:ids].split(','))
    end

    def share_params
      formatted_params(
        :id, :subject, :message, :next_state_id, :add_action, :authenticity_token,
        to: [], cc: [], bcc: [], attached_tagset_ids: [], attachments: []
      )
    end

    def evaluate_params
      formatted_params(:id, :message, :state_id, :next_state_id, :add_action, :authenticity_token, recipients: [])
    end

    def classify_params
      formatted_params(:tagset_id, :ids, :next_state_id, :add_action, :authenticity_token)
    end

    def add_doc_params
      formatted_params(:tagset_id, :next_state_id, :add_action, :authenticity_token, :regroup_files, files: [])
    end

    def form_params
      formatted_params(:state_key, :ids, :authenticity_token)
    end

    def inline_update_params
      params.require(:tagset).permit(:key, :responsible, :start_date, :end_date, accounted: [])
    end

    def send_to_params
      formatted_params(:id, :state_id, :message, :authenticity_token, recipients: [])
    end

    def find_inline_value(model, key)
      if %i[start_date end_date].include?(key)
        model.send(key)&.strftime('%d/%m/%Y')
      else
        SpiritHub::ActorFormatter.actor_ids_to_sentence(model.send(key), without_indicator: true)
      end
    end

    def request_signature_params
      formatted_params(:id, :authenticity_token, :doc_id, :comments, recipients: [])
    end

    def process_files(files, tagset_id, regroup_files)
      return [create_tagset_with_file(files, tagset_id)] if files.length < 2 || regroup_files == 'true'

      files.map { |file| create_tagset_with_file(file, tagset_id) }
    end

    def create_tagset_with_file(file, tagset_id)
      model = Tagset.create_tagset_doc_from_file(file, @user)
      model.add_tagset_doc_to_object(tagset_id, @user)
      model
    end

    def validate_or_refuse_turbo_actions(model)
      case route_controller_from_referer
      when 'spirit_hub/todos'
        if !model.is_accounted?(@user.id.to_s)
          @turbo_actions << turbo_stream.remove("tagset_#{model.id}")
        else
          update_turbo_action_current_state(model)
        end
      when 'spirit_hub/tagsets'
        @turbo_actions << turbo_stream.remove('doc_validate_refuse')
      else
        update_turbo_action_current_state(model)
      end
    end

    def send_to_turbo_actions(model)
      if route_controller_from_referer == 'spirit_hub/todos' && !model.is_accounted?(@user.id.to_s)
        @turbo_actions << turbo_stream.remove("tagset_#{model.id}")
      else
        update_turbo_action_current_state(model)
      end
    end

    def classify_turbo_actions(models)
      case route_controller_from_referer
      when 'spirit_hub/to_classify'
        models.each do |model|
          @turbo_actions << turbo_stream.remove("tagset_#{model.id}")
        end
        @turbo_actions << turbo_stream.custom_action(:change_counter, target: :end_index, attributes: { amount: -models.length })
      else
        models.each do |model|
          update_turbo_action_current_state(model)
        end
      end
    end

    def turbo_action_for_add_doc(models)
      case route_controller_from_referer
      when 'spirit_hub/favorites'
        @counts = SpiritHub::FavoritesCounts.new(@user).counts
        @turbo_actions << turbo_stream.replace('favorites-counts', partial: 'spirit_hub/favorites/counts', formats: [:html])
      when 'spirit_hub/todos'
        models.each { |model| handle_todos_add_doc(model) }
      when 'spirit_hub/to_classify'
        models.each { |model| handle_to_classify_add_doc(model) }
      else
        []
      end
    end

    def handle_todos_add_doc(model)
      return [] if model.reload.parent_tagset_ids.blank?

      # TODO : Refactor this
      referer_controller = SpiritHub::TodosController.new
      referer_controller.instance_variable_set(:@user, @user)
      referer_controller.set_columns
      @user_column_names = referer_controller.instance_variable_get(:@user_column_names)
      @user_favorites = []

      partial = referer_controller.set_view_type == 'table' ? 'spirit_hub/todos/row' : 'spirit_hub/todos/card'
      @turbo_actions << turbo_stream.prepend('todos-table-body', partial:, formats: [:html], locals: { model: })
    end

    def handle_to_classify_add_doc(model)
      return [] unless model.reload.parent_tagset_ids.blank?

      # TODO : Refactor this
      referer_controller = SpiritHub::TodosController.new
      referer_controller.instance_variable_set(:@user, @user)
      referer_controller.set_columns
      @user_column_names = referer_controller.instance_variable_get(:@user_column_names)
      @user_favorites = []

      partial = referer_controller.set_view_type == 'table' ? 'spirit_hub/to_classify/row' : 'spirit_hub/to_classify/card'
      @turbo_actions << turbo_stream.prepend('to-classify-table-body', partial:, formats: [:html], locals: { model: })
    end
  end
end
